%Kompressionspunkt durch Eintonmessung bestimmen
%
%   01.01.2021          T. Weber    

close all;
clear;

specan = 'none'; % SELECT YOUR SPECTRUM ANALYZER HERE
src = 'none'; % SELECT YOUR SIGNAL GENERATOR HERE

rbw = 0; % minimale Auflsebandbreite fr groen Dynamikbereich
t = 1; % wegen kleiner Auflsebandbreite hinreichend lange Messdauer
gain = 1; % weder Vorverstrker noch Dmpfungsglied einschalten
calpowerdbm = -30;
powerdbm = -30:1:10; % Power Sweep
frequency = 500e6; % Generatorfrequenz

% Verbindungen zu den Messgerten aufbauen
obj_specan = specan_connect(specan);
obj_src = src_connect(src);

% Parameter der Leistungsmessung einstellen
[rbw,t,gain] = specan_parameters(obj_specan,rbw,t,gain);

% Kalibrierung, direkte Verbindung
questdlg('Spektralanalysator direkt mit Combiner verbinden','Kalibrierung','OK','OK');
[frequency,calpower] = src_parameters(obj_src,frequency,10^(calpowerdbm/10)*1e-3);
direkt = specan_power(obj_specan,frequency,'direkt_kp.txt');

% Kalibrierung, Dmpfungsglied
questdlg('Dmpfungsglied einfgen','Kalibrierung','OK','OK');
[frequency,calpower] = src_parameters(obj_src,frequency,10^(calpowerdbm/10)*1e-3);
daempfungsglied = specan_power(obj_specan,frequency,'daempfungsglied_kp.txt');

% Speicher allokieren
power = zeros(numel(powerdbm),1);
dut = zeros(numel(powerdbm),1);

% Messungen
questdlg('Messobjekt einfgen','Messung','OK','OK');
for n = 1:numel(powerdbm)
    [frequency,power(n)] = src_parameters(obj_src,frequency,10^(powerdbm(n)/10)*1e-3);
    dut(n) = specan_power(obj_specan,frequency,['dut_kp_' num2str(powerdbm(n)) 'dbm.txt']);
end

% Verbindungen zu den Messgerten schlieen
specan_disconnect(obj_specan);
src_parameters(obj_src,frequency,0); % Signalgenerator ausschalten
src_disconnect(obj_src);

% Fehlerkorrektur
cal_power = power*direkt/calpower;
cal_gain = dut./power*calpower/daempfungsglied;
data = [10*log10(cal_power/1e-3),10*log10(cal_gain)];
save('kp.txt','data','-ascii');

% Kompressionspunkt bestimmen
[ausgangskpdbm,eingangskpdbm] = kompressionspunkt(10*log10(cal_power/1e-3),10*log10(cal_gain));

% Gain Compression Curve zeichnen
figure;
plot(10*log10(cal_power/1e-3),10*log10(cal_gain),'b-x');
hold on;
plot(10*log10([cal_power(1) cal_power(end)]/1e-3),(10*log10(cal_gain(1))-1)*[1 1],'r-'); % -1dB
if ~isinf(eingangskpdbm)
    plot([eingangskpdbm eingangskpdbm],[-10 20],'r-');
end
if ~isinf(ausgangskpdbm)
    ausgangskpdbm % Wert ausgeben
end

% formatieren
ylim([-10 20]);
yticks(-10:5:20);
grid on;
xlabel('10log(P/1mW)/dBm');
ylabel('10log(G)/dB');
