function obj = src_connect(src)
%SRC_CONNECT Verbindung zum Signalgenerator aufbauen
%   Kommunikation erfolgt mit SCPI-Befehlen
%   input arguments
%       src             Bezeichnung des zu verwendenden Signalgenerators
%                       untersttzt werden
%                       'none'  fr Testzwecke, keine Aktionen
%                       'smc1'  National Instruments GPIB board, board index 0, 
%                               Rohde&Schwarz SMC100A mit Option K4, 
%                               primary address 27
%                       'smc2'  National Instruments GPIB board, board index 0, 
%                               Rohde&Schwarz SMC100A mit Option K4, 
%                               primary address 28
%   output arguments
%       obj             Instrument Object
%
%   Instrument Control Toolbox 
%       S. 1-23 bis 1-24
%
%   SCPI-Befehle 
%       *IDN?
%       *RST
%
%   29.12.2020          T. Weber    

switch src
    case 'none'
        % Dummy Object erzeugen
        obj.Name = 'none';
 
        disp('no signal generator');
        
    case 'smc1'
        % Verbindung herstellen
        obj = gpib('ni',0,27,'Name',src);
        fopen(obj);

        % Abfrage der Instrument Identification zum Prfen der Verbindung
        fprintf(obj,'*IDN?');
        disp(fscanf(obj));
        
        % Preset
        fprintf(obj,'*RST');

    case 'smc2'
        % Verbindung herstellen
        obj = gpib('ni',0,28,'Name',src);
        fopen(obj);

        % Abfrage der Instrument Identification zum Prfen der Verbindung
        fprintf(obj,'*IDN?');
        disp(fscanf(obj));
        
        % Preset
        fprintf(obj,'*RST');

    otherwise
    	error('unsupported signal generator');
        
end

end
