function [frequency,power] = src_parameters(obj,frequency,power)
%SRC_PARAMETERS Parameter der Signalquelle einstellen
%   Kommunikation erfolgt mit SCPI-Befehlen
%   input arguments
%       obj             Instrument Object
%       frequency       Frequenz
%       power           Leistung in Watt, 0 bedeutet ausschalten
%   output arguments
%       frequency       tatschliche Frequenz
%       power           tatschliche Leistung in Watt
%
%   Instrument Control Toolbox 
%       S. 1-23 bis 1-24
%
%   SCPI-Befehle 
%       [:SOURce<hw>]:FREQuency[:CW|FIXed] <Fixed>
%       [:SOURce<hw>]:POWer[:LEVel][:IMMediate][:AMPLitude] <Amplitude>
%       :OUTPut<hw>[:STATe] <State>
%
%   29.12.2020          T. Weber    

switch obj.Name
    case 'none'
        
    case {'smc1','smc2'}
        % Frequenz einstellen
        fprintf(obj,'FREQ %f',frequency);
        
        % tatschliche Frequenz auslesen
        fprintf(obj,'FREQ?');
        frequency = fscanf(obj,'%f');

        if power == 0
            % ausschalten
            fprintf(obj,'OUTP OFF');
        else
            % Leistung einstellen
            fprintf(obj,'POW %f',10*log10(power/1e-3)); % Watt in dBm umrechnen

            % tatschliche Leistung auslesen
            fprintf(obj,'POW?');
            power = 10^(fscanf(obj,'%f')/10)*1e-3; % dBm in Watt umrechnen
            
            % einschalten
            fprintf(obj,'OUTP ON');
        end
        
    otherwise
    	error('unsupported signal generator');
        
end

end
