%ausgangsseitige Interceptpunkte dritter Ordnung durch Zweitonmessung bestimmen
%
%   14.11.2019          T. Weber
%   01.01.2021          Fernsteuerung der Signalgeneratoren hinzugefgt
%   13.03.2023          auch Harmonische bercksichtigt

close all;
clear;

specan = 'none'; % SELECT YOUR SPECTRUM ANALYZER HERE
src1 = 'none'; % SELECT YOUR FIRST SIGNAL GENERATOR HERE
src2 = 'none'; % SELECT YOUR SECOND SIGNAL GENERATOR HERE

rbw = 0; % minimale Auflsebandbreite fr groen Dynamikbereich
t = 1; % wegen kleiner Auflsebandbreite hinreichend lange Messdauer
gain = 1; % weder Vorverstrker noch Dmpfungsglied einschalten
calpowerdbm = -30;
powerdbm = -30:1:10; % Power Sweep
f1 = 475e6; % erste Generatorfrequenz
f2 = 525e6; % zweite Generatorfrequenz

% Messfrequenzen
frequency = [f1; f2; 3*f2; 2*f2-f1; 2*f1-f2; 3*f1]; 

% Verbindungen zu den Messgerten aufbauen
obj_specan = specan_connect(specan);
obj_src1 = src_connect(src1);
obj_src2 = src_connect(src2);

% Parameter der Leistungsmessung einstellen
[rbw,t,gain] = specan_parameters(obj_specan,rbw,t,gain);

% Kalibrierung, direkte Verbindung
questdlg('Spektralanalysator direkt mit Combiner verbinden','Kalibrierung','OK','OK');
[f1,calpower] = src_parameters(obj_src1,f1,10^(calpowerdbm/10)*1e-3);
direkt = specan_power(obj_specan,f1,'direkt_ip.txt');

% Kalibrierung, Dmpfungsglied
questdlg('Dmpfungsglied einfgen','Kalibrierung','OK','OK');
[f1,calpower] = src_parameters(obj_src1,f1,10^(calpowerdbm/10)*1e-3);
daempfungsglied = specan_power(obj_specan,f1,'daempfungsglied_ip.txt');

% Speicher allokieren
power1 = zeros(numel(powerdbm),1);
power2 = zeros(numel(powerdbm),1);
dut = zeros(numel(powerdbm),numel(frequency));

% Messungen
questdlg('Messobjekt einfgen','Messung','OK','OK');
for n = 1:numel(powerdbm)
    [f1,power1(n)] = src_parameters(obj_src1,f1,10^(powerdbm(n)/10)*1e-3);
    [f2,power2(n)] = src_parameters(obj_src2,f2,10^(powerdbm(n)/10)*1e-3);
    dut(n,:) = specan_power(obj_specan,frequency,['dut_ip_' num2str(powerdbm(n)) 'dbm.txt']);
end

% Verbindungen zu den Messgerten schlieen
specan_disconnect(obj_specan);
src_parameters(obj_src1,f1,0); % ersten Signalgenerator ausschalten
src_disconnect(obj_src1);
src_parameters(obj_src2,f2,0); % zweiten Signalgenerator ausschalten
src_disconnect(obj_src2);

% Fehlerkorrektur
cal_power = sqrt(power1.*power2)*direkt/calpower; % ist fr die Interceptpunkte irrelevant
cal_dut = dut*direkt/daempfungsglied; % das Dmpfungsglied wurde nur bei f1 kalibriert

% Interceptpunkte berechnen
[ip30,ip31,ip32,ip33] = interceptpunkte(cal_dut);
data = [10*log10(cal_power/1e-3),10*log10(ip30(:)/1e-3),...
    10*log10(ip31(:)/1e-3),10*log10(ip32(:)/1e-3),10*log10(ip33(:)/1e-3)];
save('ip.txt','data','-ascii');

% ausgangsseitige Interceptpunkte zeichnen
figure;
plot(10*log10(cal_power/1e-3),10*log10(ip30/1e-3),'b-s'); % in dBm
hold on;
plot(10*log10(cal_power/1e-3),10*log10(ip31/1e-3),'b-+'); % in dBm
plot(10*log10(cal_power/1e-3),10*log10(ip32/1e-3),'b-x'); % in dBm
plot(10*log10(cal_power/1e-3),10*log10(ip33/1e-3),'b-o'); % in dBm

% formatieren
ylim([0 30]);
yticks(0:10:30);
grid on;
ylabel('10log(P_{IP3}/1mW)/dBm');
xlabel('10log(P/1mW)/dBm');
legend('IP_{30}','IP_{31}','IP_{32}','IP_{33}','Location','northwest');
